__version__ = 0.1.2__title__ = har2jicase__description__ = 将浏览器录制的har文件为极星测试框架的YAML测试用例__url__ = __version__ = 0.1.2__author__ = Captain Ji__author_email__ = qing.ji@extremevision.com.cn__license__ = MIT


# 导入yaml和其他需要的库
import json
import time
import sys

import yaml
import re
from haralyzer import HarParser
from loguru import logger

from har_to_case.common.tools import dump_yaml

logger.remove()  # 删去import logger之后自动产生的handler，不删除的话会出现重复输出的现象
handler_id = logger.add(sys.stderr, level="INFO")  # 添加一个可以修改控制的handler


class ParseHar:
    regex_resource = re.compile(r"\/[^\/?\/]*\.(css|ico|jpg|png|gif|bmp|wav|js|jpeg|woff2)(\?.*)?")
    regex_name = re.compile(r"(?<=/api\/)(.*)(?=\?)|(?<=/api\/)(.*)")
    regex_api = re.compile(r"(\/api\/.*)")

    def __init__(self, har_file_path, output_file_path):
        '''
        ParseHar初始化，传入har文件路径，与输出文件路径
        :param har_file_path: har文件路径
        :param output_file_path: 输出文件路径
        '''
        with open(har_file_path, "r", encoding="utf-8") as f:
            self.har_parser = HarParser(yaml.safe_load(f))
        self.output_file_path = output_file_path

    def _make_request_name(self, entry: dict, test_case_dict: dict):
        '''
        解析har文件中entry项拼接出用例名，实际上是通过解析url后替换”/“为”-“生成用例名
        :param entry: har文件中entry项
        :return: 根据url拼接用例名
        '''
        global names
        if self.regex_resource.search(entry["request"]["url"]) is None:
            # 通过正则匹配url中的接口地址来取名
            names = self.regex_name.findall(entry["request"]["url"])
            try:
                for i in names[0]:
                    if i != '':
                        name = i.replace("/", '-')
                        test_case_dict['name'] = name
                        logger.info(f'成功解析用例名:{name}')
            except IndexError as e:
                logger.debug(e)
                logger.debug(names)
                pass

    def _make_request_url(self, entry: dict, request_dict: dict):
        '''
        解析har文件获取接口请求地址
        :param entry: har文件得entry部分
        :param request_dict: 构造的request字典
        :return:
        '''
        url = self.regex_api.findall(entry["request"]["url"])
        try:
            request_dict['url'] = url[0]
            logger.info(f'成功解析请求地址:{url[0]}')
        except IndexError as e:
            logger.debug(e)
            logger.debug(url)
            pass

    def _make_request_method(self, entry: dict, request_dict: dict):
        '''
        解析har文件获取请求方法
        :param entry: har文件得entry部分
        :param request_dict: 构造的request字典
        :return:
        '''
        request_dict['method'] = entry["request"]["method"]
        logger.info(f"成功解析请求方法:{request_dict['method']}")

    def _make_request_header(self, request_dict: dict):
        '''
        构造request字典中headers部分;框架中有独立的请求头生成方法,所以此工具只负责生成固定的请求头,不解析entry获取请求头信息
        :param request_dict:
        :return:
        '''

        request_dict['headers'] = {'Authorization': "", "Content-Type": "application/json; charset=utf-8"}
        logger.info(f"成功配置请求头:{request_dict['headers']}")

    def _make_request_params(self, entry: dict, request_dict: dict):
        '''
        解析har文件获取请求参数
        :param entry: har文件的entry部分
        :param request_dict: 构造的request字典
        :return:
        '''
        if "postData" in entry["request"]:
            request_dict['params'] = json.loads(entry["request"]["postData"]["text"])
            logger.info(f"成功解析请求参数:{request_dict['params']}")
        else:
            request_dict['params'] = None

    def _make_respone_validate(self, entry: dict, request_dict: dict):
        '''
        解析har文件respone部分,生成用例断言
        :param entry: har文件的entry部分
        :param request_dict: 构造的request字典
        '''
        try:
            data = json.loads(entry['response']['content']['text'])
            request_dict['validate']['equals']['status_code'] = data['status_code']
            logger.info(f"成功解析返回状态码:{request_dict['validate']['equals']}")
        except KeyError as e:
            logger.debug(e)

    def _make_respone_valiade_contains(self, entry: dict, request_dict: dict):
        '''
        解析har文件respone部分,通过返回体中的key生成断言应包含的关键字
        :param entry: har文件的entry部分
        :param request_dict: 构造的request字典
        :return:
        '''
        try:
            data = json.loads(entry['response']['content']['text'])
            data = data['data']
            validate_contains = ''
            if isinstance(data, dict):
                for key in data.keys():
                    if validate_contains == '':
                        validate_contains = key
                    else:
                        validate_contains = validate_contains + '-' + key
            elif isinstance(data, str):
                validate_contains = data
            else:
                for item in data:
                    # todo: data中的data继续解析，后续看需求再定
                    if isinstance(item, dict):
                        pass
                    else:
                        if validate_contains == '':
                            validate_contains = item
                        else:
                            logger.debug(item)
                            validate_contains = validate_contains + '-' + item
            request_dict['validate']['contains'] = validate_contains
            logger.info('成功解析返回体')
        except KeyError as e:
            logger.debug(e)

    def _generate_case_dict(self, entry, test_case_dict, request_dict):
        '''
        生成用例字典的函数,将
        :param entry:
        :param test_case_dict:
        :param request_dict:
        :return:
        '''
        self._make_request_name(entry, test_case_dict)
        self._make_request_url(entry, request_dict)
        self._make_request_method(entry, request_dict)
        self._make_request_params(entry, request_dict)
        self._make_request_header(request_dict)
        self._make_respone_validate(entry, request_dict)
        self._make_respone_valiade_contains(entry, request_dict)

    def _generate_case_list(self) -> list:
        # 遍历har文件中的每个请求
        test_case_list = []
        for page in self.har_parser.pages:
            for entry in page.entries:
                # 提取需要的信息
                if self.regex_resource.search(entry["request"]["url"]) is None:
                    request_dict = {
                        "url": '',
                        "method": '',
                        "headers": '',
                        # "cookies": cookies,
                        "params": '',
                        "validate": {
                            'equals': {
                                'status_code': ''
                            },
                            'contains': ''
                        }
                    }
                    test_case_dict = {"name": '',
                                      "requests": request_dict}
                    # 构建框架用例格式字典
                    logger.info(
                        '---------------------------------------构建用例--------------------- -----------------')
                    self._generate_case_dict(entry, test_case_dict, request_dict)
                    if test_case_dict['name'] == '':
                        pass
                    else:
                        test_case_list.append(test_case_dict)
            return test_case_list

    def generate_yaml_case(self):
        '''
        生成yaml格式用例
        '''
        now = time.strftime("%Y-%m-%d-%H_%M_%S", time.localtime(time.time()))
        if '.yaml' in self.output_file_path or '.yml' in self.output_file_path:
            file = self.output_file_path

        else:
            file = self.output_file_path + f"/record_{now}.yaml"
        test_case = self._generate_case_list()
        dump_yaml(test_case, file)


if __name__ == '__main__':
    case = ParseHar('./192.168.1.117.har', './casefile')
    case.generate_yaml_case()



#! /usr/bin/env python
"""
-*- coding: UTF-8 -*-
Project   : har2case
Author    : Captain
Email     : qing.ji@extremevision.com.cn
Date      : 2023/4/19 14:15
FileName  : main.py
Software  : PyCharm
Desc      : 项目主入口
"""
import sys
import argparse
import time

from loguru import logger
from har_to_case.parse_har import ParseHar
from har_to_case import __description__

try:
    from har_to_case import __version__ as version
except ImportError:
    version = None

if len(sys.argv) == 1:
    sys.argv.append('--help')
now = time.strftime("%Y-%m-%d-%H_%M_%S", time.localtime(time.time()))
parser = argparse.ArgumentParser(description=__description__)
# parser.add_argument('-v', '--version', help=f"显示版本;当前版本：{version}", action="store_true")
parser.add_argument('-i', '--input', type=str, help='har文件路径（必填）')
parser.add_argument('-o', '--output', default=f"record_{now}.yaml", type=str, help='指定输出文件名')


def main():
    args = parser.parse_args()
    if args.version:
        logger.info(f"当前版本：{version}")
        exit(0)
    arguments = parser.parse_args()
    if not arguments.input:
        logger.error("har文件路径为必填项：-i {har file path}")
    ParseHar(arguments.input, arguments.output).generate_yaml_case()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
-*- coding: utf-8 -*-
Author   : JiQing
Email    : qing.ji@extremevision.com.cn
Date     : 2022/10/24 10:07
Desc     :
FileName : tools.py
Software : PyCharm
"""

import io
import json
import os

import yaml
from loguru import logger


def get_file_path(url, entry_json):
    '''
    获取用例文件地址
    :param url:
    :param entry_json:
    :return:
    '''
    file_name = 'test_' + ''.join(url.split("/")[-1])
    api = is_file_exist(os.path.join(os.getcwd(), "api"))
    module = is_file_exist(
        os.path.join(
            api, {"module": v.get("tags")[0] for k, v in entry_json.items()}.get("module")))
    return os.path.join(module, file_name)


def is_file_exist(files):
    '''
    检查文件是否存在
    :param files:
    :return:
    '''
    if not os.path.exists(files):
        os.mkdir(files)
    return files


def dump_yaml(case, yaml_file):
    '''
    创建YAML格式文件
    :param case:
    :param yaml_file:
    :return:
    '''
    if os.path.exists(yaml_file):
        logger.debug("文件存在，跳过解析")
        return

    logger.info("开始创建YAML格式文件")
    with io.open(yaml_file, 'w', encoding="utf-8") as outfile:
        yaml.dump(case, outfile, allow_unicode=True, default_flow_style=False, indent=4)

        logger.info(f"yaml文件:{yaml_file}创建成功")


def dump_json(case, json_file):
    '''
    创建YAML格式文件
    :param case:
    :param json_file:
    :return:
    '''
    logger.info("开始创建json格式文件")
    if not os.path.exists(json_file):
        os.makedirs(json_file)

    with io.open(json_file, 'w', encoding="utf-8") as outfile:
        my_json_str = json.dumps(case, ensure_ascii=False, indent=4)
        if isinstance(my_json_str, bytes):
            my_json_str = my_json_str.decode("utf-8")
        outfile.write(my_json_str)
    logger.info(f"json文件: {json_file}创建成功")


def get_target_value(key, _json, tmp_list=None):
    '''
    从字典中获取目标值
    :param key:
    :param _json:
    :param tmp_list:
    :return:
    '''
    if tmp_list is None:
        tmp_list = []

    if not isinstance(_json, dict) or not isinstance(tmp_list, list):
        return 'args[1]不是字典或args[-1]不是列表'

    if key in _json.keys():
        tmp_list.append(_json[key])

    for value in _json.values():
        if isinstance(value, dict):
            get_target_value(key, value, tmp_list)
        elif isinstance(value, (list, tuple)):
            _get_list_value(key, value, tmp_list)

    return tmp_list


def _get_list_value(key, value, tmp_list):
    '''
    循环访问列表获取目标值
    :param key:
    :param value:
    :param tmp_list:
    :return:
    '''
    for val_ in value:
        if isinstance(val_, dict):
            get_target_value(key, val_, tmp_list)
        elif isinstance(val_, (list, tuple)):
            _get_list_value(key, val_, tmp_list)



#! /usr/bin/env python
"""
-*- coding: UTF-8 -*-
Project   : pypi
Author    : Captain
Email     : qing.ji@extremevision.com.cn
Date      : 2023/4/21 15:34
FileName  : setup.py
Software  : PyCharm
Desc      :
            手动构建与上传pypi
            python setup_backup.py sdist bdist_wheel
            python -m twine upload dist/* -u jiqing19861123 -p CJ2938cj

            自动构建与上传pypi
            python setup.py upload
"""

import io
import os
import sys
from shutil import rmtree
from setuptools import find_packages, setup, Command

# 写入自己的内容
NAME = 'har2jicase'  # 项目名
DESCRIPTION = '将浏览器录制的har文件为极星测试框架的YAML测试用例'
KEYWORDS = 'har json compare comparison case yaml yml'
AUTHOR = 'Captain Ji'
EMAIL = 'qing.ji@extremevision.com.cn'
URL = ''  # 项目git地址
LICENSE = 'MIT'  # 项目license
PACKAGES = find_packages(include=('har*',))  # 项目包含的包exclude为排除的包
REQUIRES_PYTHON = '>=3.6.13'
PROJECT_PATH = 'har_to_case'
VERSION = '0.1.2'  # 为项目指定目前的版本号
pypi_username = 'jiqing19861123'
pypi_password = 'CJ2938cj'
# 项目依赖的库
REQUIRED = ['loguru~=0.6.0', 'haralyzer~=2.2.0', 'PyYAML~=6.0']
# 项目入口文件
ENTRY_POINTS = {
    'console_scripts': [
        f'har2case = {PROJECT_PATH}.main:main'
    ]
}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        LONG_DESC = '\n' + f.read()
except FileNotFoundError:
    LONG_DESC = DESCRIPTION

about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(f"{here}/{PROJECT_PATH}", project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

with open(os.path.join(f"{here}/{PROJECT_PATH}", '__init__.py'), 'w', encoding='utf-8') as f:
    f.write(f'__version__ = {VERSION}')
    f.write(f'__title__ = {NAME}'
            f'__description__ = {DESCRIPTION}'
            f'__url__ = {URL}'
            f'__version__ = {VERSION}'
            f'__author__ = {AUTHOR}'
            f'__author_email__ = {EMAIL}'
            f'__license__ = {LICENSE}')


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system(f'twine upload dist/* -u {pypi_username} -p {pypi_password}')

        # self.status('Pushing git tags…')
        # os.system('git tag v{0}'.format(about['__version__']))
        # os.system('git push --tags')

        sys.exit()


setup(
    name=NAME,  # 和前边的保持一致
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=LONG_DESC,  # 默认是readme文件。
    long_description_content_type='text/markdown',
    keywords=KEYWORDS,
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    license=LICENSE,
    packages=PACKAGES,
    include_package_data=True,
    zip_safe=True,
    install_requires=REQUIRED,
    entry_points=ENTRY_POINTS,
    cmdclass={
        'upload': UploadCommand,
    }
)
